package com.gx.obe.template.composite;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.IntStream;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.DictEnum;
import com.gx.obe.template.enumeration.TemplateTypeEnum;
import com.gx.obe.web.entity.DictionaryValue;
import com.gx.obe.web.service.BaseDictionaryValueService;

public class ProjectTemplateFormComposite extends Composite {
	
	public final BaseDictionaryValueService dictionaryValueService = new BaseDictionaryValueService();
	
	private Combo templateTypeCombo;
	private Button defaultTypeButton;
	private Composite agentBtnCheckComposite;
	private Button fixedFactorButton;
	
	private Map<String, Button> agentBtnCheckMap = new HashMap<>();
	
	/**
	 * Create the composite.
	 * 
	 * @param parent
	 * @param style
	 */
	public ProjectTemplateFormComposite(Composite parent, int style) {
		super(parent, style);
		this.setLayout(new GridLayout(2, false));
		
		CLabel lblNewLabel = new CLabel(this, SWT.NONE);
		lblNewLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblNewLabel.setText("模板类型：");
		
		templateTypeCombo = new Combo(this, SWT.READ_ONLY);
		templateTypeCombo.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		CLabel lblNewLabel_3 = new CLabel(this, SWT.NONE);
		lblNewLabel_3.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblNewLabel_3.setText("默认模板：");
		
		defaultTypeButton = new Button(this, SWT.CHECK);
		defaultTypeButton.setText("默认模板");
		
		CLabel lblNewLabel_2 = new CLabel(this, SWT.NONE);
		lblNewLabel_2.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, false, false, 1, 1));
		lblNewLabel_2.setText("代理机构：");
		
		agentBtnCheckComposite = new Composite(this, SWT.NONE);
		RowLayout rl_composite_2 = new RowLayout(SWT.HORIZONTAL);
		rl_composite_2.marginLeft = 0;
		agentBtnCheckComposite.setLayout(rl_composite_2);
		agentBtnCheckComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		CLabel lblNewLabel_1 = new CLabel(this, SWT.NONE);
		lblNewLabel_1.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblNewLabel_1.setText("模板分类：");
		
		fixedFactorButton = new Button(this, SWT.CHECK);
		fixedFactorButton.setText("定档模板");
		
		init();
		
	}
	
	private void init() {
		initTemplateTypeCombo();
		initAgentCheckButton();
	}
	
	/**
	 * @Description: 初始化模版类型下拉框
	 * @author chenxw
	 */
	private void initTemplateTypeCombo() {
		TemplateTypeEnum[] templateTypeEnums = TemplateTypeEnum.values();
		IntStream.range(0, templateTypeEnums.length).forEach(i -> {
			templateTypeCombo.add(templateTypeEnums[i].getValue(), i);
			templateTypeCombo.setData(i + "", templateTypeEnums[i].getKey());
		});
	}
	
	/**
	 * @Description: 初始化代理机构复选框
	 * @author chenxw
	 */
	private void initAgentCheckButton() {
		List<DictionaryValue> tenderMethodList = dictionaryValueService.getDictValueListByDicCode(DictEnum.AGENT);
		if (Objects.isNull(tenderMethodList)) {
			return;
		}
		for (DictionaryValue dictionaryValue : tenderMethodList) {
			createAgentCheckButton(dictionaryValue.getDictValueName());
		}
	}
	
	/**
	 * @Description: 生成代理复选框
	 * @author chenxw
	 * @param text
	 */
	private void createAgentCheckButton(String text) {
		Button btnCheckButton = new Button(agentBtnCheckComposite, SWT.CHECK);
		btnCheckButton.setText(text);
		agentBtnCheckMap.put(text, btnCheckButton);
	}
	
	public Combo getTemplateTypeCombo() {
		return templateTypeCombo;
	}
	
	public Button getFixedFactorButton() {
		return fixedFactorButton;
	}
	
	public Map<String, Button> getAgentBtnCheckMap() {
		return agentBtnCheckMap;
	}
	
	public Button getDefaultTypeButton() {
		return defaultTypeButton;
	}
	
	@Override
	protected void checkSubclass() {
		// Disable the check that prevents subclassing of SWT components
	}
	
}
