package com.gx.obe.template.composite;

import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.FileDialogUtils;
import org.eclipse.swt.core.utils.WidgetUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;

import com.gx.obe.common.widget.composite.ManageComposite;
import com.gx.obe.template.agent.ProjectTemplateAgent;
import com.gx.obe.template.dialog.ProjectTemplateFormDialog;
import com.gx.obe.template.entity.ProjectTemplate;
import com.gx.obe.template.enumeration.TemplateTypeEnum;
import com.gx.obe.template.service.ProjectTemplateService;
import com.gx.obe.template.utils.TemplateUtil;
import com.gx.obe.template.vo.ProjectTypeVo;
import com.gx.obe.util.utils.StringUtils;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 项目模板管理界面
 * @author chenxw
 */
public class ProjectTemplateManageComposite extends ManageComposite<ProjectTemplate> {
	
	public final ProjectTemplateService projectTemplateService = new ProjectTemplateService();
	
	private ProjectTemplateFormComposite formComposite;
	
	public ProjectTemplateManageComposite(Composite parent, int style) {
		super(parent, style);
		
		setTitleName("模板列表");
		
		GridColumn gridColumn_1 = new GridColumn(grid, SWT.NONE);
		gridColumn_1.setText("模板名称");
		gridColumn_1.setWidth(150);
		addColumnTextListener(0, ProjectTemplate::getLocalTemplateName);
		
		GridColumn gridColumn = new GridColumn(grid, SWT.CENTER);
		gridColumn.setText("模板类型");
		gridColumn.setWidth(150);
		addColumnTextListener(1, this::getTemplateType);
		
		GridColumn gridColumn_2 = new GridColumn(grid, SWT.CENTER);
		gridColumn_2.setText("是否为默认模板");
		gridColumn_2.setWidth(150);
		addColumnTextListener(2, this::getDefaultType);
	}
	
	/**
	 * @Description: 获取模板类型
	 * @author chenxw
	 * @param projectTemplate
	 * @return 
	 */
	private String getTemplateType(ProjectTemplate projectTemplate) {
		return Optional.of(projectTemplate)
				.map(ProjectTemplate::getTemplateType)
				.map(TemplateTypeEnum::getByKey)
				.map(TemplateTypeEnum::getValue)
				.orElse("");
	}
	
	/**
	 * @Description: 获取是否定档
	 * @author chenxw
	 * @param projectTemplate
	 * @return 
	 */
	private String getDefaultType(ProjectTemplate projectTemplate) {
		return Optional.of(projectTemplate)
				.map(ProjectTemplate::getProjectType)
				.map(ProjectTemplateAgent::getProjectType)
				.map(ProjectTypeVo::isDefaultType)
				.map(t -> t ? "是" : "否")
				.orElse("");
	}
	
	@Override
	protected void initLowerComposite() {
		((GridLayout) lowerComposite.getLayout()).numColumns = 3;
		
		Composite composite = new Composite(lowerComposite, SWT.NONE);
		composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridData gd_composite = new GridData(SWT.FILL, SWT.FILL, false, false, 1, 1);
		gd_composite.widthHint = 400;
		composite.setLayoutData(gd_composite);
		composite.setLayout(new GridLayout(1, false));
		
		formComposite = new ProjectTemplateFormComposite(composite, SWT.NONE);
		formComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false, 1, 1));
		
		Composite composite_1 = new Composite(composite, SWT.NONE);
		composite_1.setLayout(new GridLayout(1, false));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		EButton btnNewButton = new EButton(composite_1, SWT.NONE);
		btnNewButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				queryAction();
			}
		});
		GridData gd_btnNewButton = new GridData(SWT.CENTER, SWT.CENTER, true, false, 1, 1);
		gd_btnNewButton.heightHint = 30;
		gd_btnNewButton.widthHint = 90;
		btnNewButton.setLayoutData(gd_btnNewButton);
		btnNewButton.setText("查询");
		
		Label lblNewLabel_3 = new Label(lowerComposite, SWT.NONE);
		lblNewLabel_3.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		GridData gd_lblNewLabel_3 = new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1);
		gd_lblNewLabel_3.widthHint = 5;
		lblNewLabel_3.setLayoutData(gd_lblNewLabel_3);
	}
	
	private void queryAction() {
		refresh();
	}

	@Override
	protected List<ProjectTemplate> initTList() {
		String templateType = (String) WidgetUtils.getComboSelection(formComposite.getTemplateTypeCombo());
		boolean fixedFactor = formComposite.getFixedFactorButton().getSelection();
		boolean defaultType = formComposite.getDefaultTypeButton().getSelection();
		List<String> keyList = formComposite.getAgentBtnCheckMap().entrySet().stream().filter(e -> e.getValue().getSelection()).map(Map.Entry::getKey).collect(Collectors.toList());
		return ProjectTemplateAgent.initAllList()
				.filter(ProjectTemplate::getTemplateType, t -> templateType == null || templateType.equals(t))
				.filterType(ProjectTypeVo::isFixedFactor, t -> fixedFactor == t)
				.filterType(ProjectTypeVo::isDefaultType, t -> defaultType == t)
				.filterType(ProjectTypeVo::getAgentKeyList, t -> keyList.isEmpty() || (Objects.nonNull(t) && keyList.stream().anyMatch(t::contains)))
				.sorted(TemplateUtil.COMPARATOR)
				.get();
	}
	
	@Override
	protected void add() {
		String fileName = FileDialogUtils.getFileNames(SWT.SINGLE, "请选模板文件", new String[] { "*docx;*.xlsx" });
		if (StringUtils.isEmpty(fileName)) {
			return ;
		}
		ProjectTemplateFormDialog dialog = new ProjectTemplateFormDialog(getShell(), fileName, getEntityListener());
		dialog.open();
	}
	
	@Override
	protected void edit(ProjectTemplate t) {
		ProjectTemplateFormDialog dialog = new ProjectTemplateFormDialog(getShell(), t, getEntityListener());
		dialog.open();
	}
	
	@Override
	protected boolean delete(ProjectTemplate t) {
		if (!MessageDialog.openConfirm(getShell(), "删除模板文件", "确认删除当前模板文件吗？", "确认", "取消")) {
			return false;
		}
		if (!projectTemplateService.deleteProjectTemplate(t)) {
			MessageDialog.openError(getShell(), "删除失败！");
			return false;
		}
		MessageDialog.openShortMessage("删除成功！");
		return true;
	}
}
