package com.gx.obe.template.dialog;

import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.WidgetUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.common.file.upload.UploadFileUtil;
import com.gx.obe.common.file.upload.infer.UploadFileAdapter;
import com.gx.obe.common.widget.listener.EntityListener;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.message.Messages;
import com.gx.obe.template.agent.ProjectTemplateAgent;
import com.gx.obe.template.composite.ProjectTemplateFormComposite;
import com.gx.obe.template.entity.ProjectTemplate;
import com.gx.obe.template.service.ProjectTemplateService;
import com.gx.obe.template.utils.ProjectTemplateUtils;
import com.gx.obe.template.vo.ProjectTypeVo;

/**
 * @Description: 项目模板表单界面
 * @author chenxw
 */
public class ProjectTemplateFormDialog extends EDialog {
	
	public final ProjectTemplateService projectTemplateService = new ProjectTemplateService();
	
	private ProjectTemplate projectTemplate;
	
	private boolean isNew;
	
	private ProjectTemplateFormComposite formComposite;
	
	private ProjectTypeVo projectType;
	
	private EntityListener<ProjectTemplate> entityListener;
	
	/**
	 * @author chenxw
	 * @param parentShell
	 * @wbp.parser.constructor
	 */
	public ProjectTemplateFormDialog(Shell parentShell, String templatePath , EntityListener<ProjectTemplate> entityListener) {
		super(parentShell);
		this.projectTemplate = ProjectTemplateUtils.create(templatePath);
		this.setTitle("新增模板");
		this.isNew = true;
		this.entityListener = entityListener;
	}
	
	/**
	 * @author chenxw
	 * @param parentShell
	 * @param projectTemplate
	 */
	public ProjectTemplateFormDialog(Shell parentShell, ProjectTemplate projectTemplate , EntityListener<ProjectTemplate> entityListener) {
		super(parentShell);
		this.projectTemplate = Optional.of(projectTemplate).get();
		this.setTitle("修改模板");
		this.isNew = false;
		this.entityListener = entityListener;
	}
	
	/**
	 * @Description:
	 * @author chenxw
	 * @param parent
	 * @return
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout(1, false));
		
		formComposite = new ProjectTemplateFormComposite(container, SWT.NONE);
		formComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setLayout(new GridLayout(2, false));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		EButton btnNewButton = new EButton(composite_1, SWT.NONE);
		btnNewButton.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				saveAction();
			}
		});
		btnNewButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, true, false, 1, 1));
		btnNewButton.setText(Messages.Button_save);
		
		EButton btnNewButton_1 = new EButton(composite_1, SWT.NONE);
		btnNewButton_1.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				close();
			}
		});
		btnNewButton_1.setText("取消");
		
		init();
		
		return container;
	}
	
	/**
	 * @Description: 初始化
	 * @author chenxw
	 */
	private void init() {
		projectType = Optional.of(projectTemplate).map(ProjectTemplate::getProjectType).map(ProjectTemplateAgent::getProjectType).orElseGet(ProjectTypeVo::new);
		initData();
	}
	
	/**
	 * @Description: 初始化数据
	 * @author chenxw
	 */
	private void initData() {
		if (isNew) return;
		WidgetUtils.setComboSelection(formComposite.getTemplateTypeCombo(), projectTemplate.getTemplateType());
		formComposite.getFixedFactorButton().setSelection(projectType.isFixedFactor());
		formComposite.getDefaultTypeButton().setSelection(projectType.isDefaultType());
		formComposite.getAgentBtnCheckMap().entrySet().stream().filter(e -> agentKeyListContains(e.getKey())).map(Map.Entry::getValue).forEach(t -> t.setSelection(true));
	}
	
	/**
	 * @Description: 代理机构编码列表中是否含有此编码
	 * @author chenxw
	 * @param key
	 * @return
	 */
	private boolean agentKeyListContains(String key) {
		return Optional.of(projectType).map(ProjectTypeVo::getAgentKeyList).map(t -> t.contains(key)).orElse(false);
	}
	
	/**
	 * @Description: 保存事件
	 * @author chenxw
	 */
	private void saveAction() {
		projectType.setFixedFactor(formComposite.getFixedFactorButton().getSelection());
		projectType.setDefaultType(formComposite.getDefaultTypeButton().getSelection());
		projectType.setAgentKeyList(formComposite.getAgentBtnCheckMap().entrySet().stream().filter(e -> e.getValue().getSelection()).map(Map.Entry::getKey).collect(Collectors.toList()));
		projectTemplate.setProjectType(JsonUtil.jsonObjectToStr(projectType));
		projectTemplate.setTemplateType((String) WidgetUtils.getComboSelection(formComposite.getTemplateTypeCombo()));
		
		if (!isNew) {
			afterSave(projectTemplateService.updateAssignProperty(projectTemplate, new String[] { "projectType", "templateType" }));
			return;
		}
		UploadFileUtil uploadFileUtil = new UploadFileUtil(projectTemplate.getFile().getAbsolutePath(), projectTemplate.getTemplateFileName(), new UploadFileAdapter() {
			public void Completed(boolean loginFlag) {
				Display.getDefault().syncExec(() -> {
					if (!loginFlag) {
						MessageDialog.openError(getShell(), "模板文件上传失败！");
						return;
					}
					afterSave(projectTemplateService.insert(projectTemplate));
				});
			}
		});
		uploadFileUtil.startUploadFileFile();
	}
	
	/**
	 * @Description: 保存之后
	 * @author chenxw
	 * @param save
	 */
	private void afterSave(boolean save) {
		if (!save) {
			MessageDialog.openError(getShell(), Messages.Tips_saveFail);
			return;
		}
		close();
		if(null != entityListener) {
			entityListener.addEntity(projectTemplate);
		}
	}
	
	
	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(446, 512);
	}
	
}
