package com.gx.obe.template.service;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;

import com.gx.obe.components.core.Constants;
import com.gx.obe.http.json.JsonUtil;
import com.gx.obe.template.entity.ProjectTemplate;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.UuidUtils;
import com.gx.obe.web.utils.NetworkRequest;
import com.gx.obe.web.utils.ResultStatus;
import com.gx.obe.web.vo.UpdateAssignPropertyVo;
import com.gx.utils.NowTimeUtils;

public class ProjectTemplateService {
	
	private Logger logger = Logger.getLogger(ProjectTemplateService.class);
	private static final String URL = "projectTemplate";
	
	/**
	 * @Description: 获取所有模板
	 * @author chenxw
	 * @return
	 */
	public List<ProjectTemplate> getAllList() {
		String strWebServerUrl = Constants.getServiceUrl(URL + "/getAllList");
		try {
			String result = NetworkRequest.get(strWebServerUrl);
			if (ResultStatus.getReultStatusIsNotEmpty(result)) {
				return JsonUtil.jsonToList(result, ProjectTemplate.class);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}
	
	/**
	 * @Description: 根据项目id及模板类型获取列表
	 * @author chenxw
	 * @param tenderId
	 * @param templateType
	 * @return
	 */
	public List<ProjectTemplate> getListByTenderIdAndTemplateType(String tenderId, String templateType) {
		String strWebServerUrl = Constants.getServiceUrl(URL + "/getListByTenderIdAndTemplateType");
		Map<String, Object> param = new HashMap<String, Object>();
		param.put("tenderId", tenderId);
		param.put("templateType", templateType);
		try {
			String result = NetworkRequest.get(strWebServerUrl, param);
			if (ResultStatus.getReultStatusIsNotEmpty(result)) {
				return JsonUtil.jsonToList(result, ProjectTemplate.class);
			}
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}
	
	/**
	 * @Description: 指定字段修改
	 * @author chenxw
	 * @param projectTemplate
	 * @param attributes
	 * @return
	 */
	public boolean updateAssignProperty(ProjectTemplate projectTemplate, String[] attributes) {
		String strWebServerUrl = Constants.getServiceUrl(URL + "/updateAssignProperty");
		UpdateAssignPropertyVo<ProjectTemplate> updateAssignPropertyVo = new UpdateAssignPropertyVo<>();
		updateAssignPropertyVo.setEntity(projectTemplate);
		updateAssignPropertyVo.setAttributes(attributes);
		try {
			String result = NetworkRequest.post(strWebServerUrl, JsonUtil.jsonObjectToStr(updateAssignPropertyVo));
			return ResultStatus.getReultStatusBoolean(result);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return false;
	}
	
	/**
	 * @Description: 插入
	 * @author chenxw
	 * @param projectTemplate
	 * @return
	 */
	public boolean insert(ProjectTemplate projectTemplate) {
		String strWebServerUrl = Constants.getServiceUrl(URL + "/insert");
		projectTemplate.setId(UuidUtils.getUUID());
		projectTemplate.setModifyDate(NowTimeUtils.getNowDate());
		try {
			String result = NetworkRequest.post(strWebServerUrl, JsonUtil.jsonObjectToStr(projectTemplate));
			return ResultStatus.getReultStatusBoolean(result);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return false;
	}
	
	/**
	 * @Description: 删除项目模板
	 * @author chenxw
	 * @param projectTemplate
	 * @return
	 */
	public boolean deleteProjectTemplate(ProjectTemplate projectTemplate) {
		String strWebServerUrl = Constants.getServiceUrl(URL + "/deleteProjectTemplate");
		try {
			String result = NetworkRequest.post(strWebServerUrl, JsonUtil.jsonObjectToStr(projectTemplate));
			return ResultStatus.getReultStatusBoolean(result);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return false;
	}
	
	/**
	 * @Description: 获取最晚修改时间
	 * @author chenxw
	 * @return
	 */
	public Date getLatestModifyTime() {
		String strWebServerUrl = Constants.getServiceUrl(URL + "/getLatestModifyTime");
		try {
			return DateUtils.toDate(NetworkRequest.get(strWebServerUrl));
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return null;
	}
	
}
