package com.gx.obe.template.utils;

import java.io.File;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import com.gx.obe.common.file.download.DownloadFileUtill;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.config.ConfigConstants;
import com.gx.obe.config.utils.PropertiesUtils;
import com.gx.obe.template.agent.ProjectTemplateAgent;
import com.gx.obe.template.entity.ProjectTemplate;
import com.gx.obe.template.enumeration.TemplateFileTypeEnum;
import com.gx.obe.template.service.ProjectTemplateService;
import com.gx.obe.util.utils.DateUtils;
import com.gx.obe.util.utils.FileUtils;
import com.gx.obe.util.utils.MD5Utils;
import com.gx.utils.NowTimeUtils;

/**
 * @Description: 项目模板工具类
 * @author chenxw
 */
public class TemplateUtil {
	
	private static final ProjectTemplateService PTS = new ProjectTemplateService();
	
	private static final Map<String, File> FILE_MAP = new HashMap<>();
	
	public static final Comparator<ProjectTemplate> COMPARATOR = Comparator.comparing(ProjectTemplate::getLocalTemplateName, WinFileNameComparator.get());
	
	/**
	 * @Description: 初始化项目模板文件（含时间校验）
	 * @author chenxw
	 */
	public static void initFileOfTimeVerification() {
		Date updateTime = Optional.ofNullable(ConfigConstants.TEMPLATE_UPDATE_TIME)
				.map(PropertiesUtils::getProperty)
				.map(DateUtils::toDate)
				.orElseGet(DateUtils::getEarliestDate);
		
		boolean notUpdate = Optional.ofNullable(PTS.getLatestModifyTime())
				.map(updateTime::after)
				.orElse(true);
		
		if (notUpdate) {
			return;
		}
		initFile();
		PropertiesUtils.saveProperty(ConfigConstants.TEMPLATE_UPDATE_TIME, DateUtils.getTimestampFormat(NowTimeUtils.getNowDate()));
	}
	
	/**
	 * @Description: 初始化项目模板文件
	 * @author chenxw
	 */
	public static void initFile() {
		ProjectTemplateAgent.initAllList().forEach(TemplateUtil::downloadFile);
	}
	
	/**
	 * @Description: 下载模板文件
	 * @author chenxw
	 * @param projectTemplate
	 */
	public static void downloadFile(ProjectTemplate projectTemplate) {
		String fileName = projectTemplate.getTemplateFileName();
		File file = new File(FileConstants.SYSTEM_TEMPLATE_PATH.concat(fileName));
		if (file.exists() && projectTemplate.getTemplateId().equals(MD5Utils.getFileMD5(file))) {
			return;
		}
		new DownloadFileUtill(fileName, FileConstants.SYSTEM_TEMPLATE_PATH, null).startDownloadFile();
	}
	
	/**
	 * @Description: 读取项目的报表模板
	 * @param tenderProject
	 * @return
	 */
	public static List<ProjectTemplate> getList(String tenderId, String templateType) {
		return ProjectTemplateAgent.initProjectList(tenderId, templateType)
				.filter(TemplateUtil::getFile, File::exists)
				.peek(t -> t.setFileType(TemplateFileTypeEnum.SYSTEM.getType()))
				.peek(t -> t.setFile(getFile(t)))
				.sorted(COMPARATOR)
				.get();
	}
	
	/**
	 * @Description: 读取所有的报表模板
	 * @return
	 */
	public static List<ProjectTemplate> getAllList() {
		return ProjectTemplateAgent.initAllList()
				.sorted(COMPARATOR)
				.get();
	}
	
	/**
	 * @Description: 获取自定义模板路径
	 * @author chenxw
	 * @return 
	 */
	public static List<ProjectTemplate> getCustomList(String fileName) {
		return FileUtils.listAllFiles(FileConstants.CUSTOM_TEMPLATE_PATH)
				.stream()
				.filter(t -> fileName == null || t.getName().contains(fileName))
				.map(ProjectTemplateUtils::create)
				.sorted(COMPARATOR)
				.collect(Collectors.toList());
	}
	
	/**
	 * @Description: 获取文件
	 * @author chenxw
	 * @param projectTemplate
	 * @return 
	 */
	public static File getFile(ProjectTemplate projectTemplate) {
		return FILE_MAP.computeIfAbsent(projectTemplate.getTemplateFileName(), FunctionUtils.newFile.compose(FileConstants.SYSTEM_TEMPLATE_PATH::concat));
	}
	
}
