package com.gx.obe.template.utils;

import java.math.BigInteger;
import java.text.Collator;
import java.util.Comparator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/** 
 * @Description: 文件名称排序器
 * @author chenxw 
 */
public class WinFileNameComparator implements Comparator<String> {
	
	private static final WinFileNameComparator WIN_FILE_NAME_COMPARATOR = new WinFileNameComparator();
	
	private static final Collator CHINA_COLLATOR = Collator.getInstance();
	
	private static final Pattern PATTERN = Pattern.compile("[1-9]\\d*");
	
	private WinFileNameComparator() {}
	
	public static WinFileNameComparator get() {
		return WIN_FILE_NAME_COMPARATOR;
	}
	
	@Override
	public int compare(String o1, String o2) {
		
		int index1 = o1.lastIndexOf(".");
		int index2 = o2.lastIndexOf(".");
		
		o1 = index1 == -1 ? o1 : o1.substring(0, index1);
		o2 = index2 == -1 ? o2 : o2.substring(0, index2);
		
		Matcher matcher1 = PATTERN.matcher(o1);
		Matcher matcher2 = PATTERN.matcher(o2);
		
		int end1 = 0;
		int end2 = 0;
		
		while (true) {
			boolean find1 = matcher1.find();
			boolean find2 = matcher2.find();
			
			String s1 = o1.substring(end1, find1 ? matcher1.start() : o1.length());
			String s2 = o2.substring(end2, find2 ? matcher2.start() : o2.length());
			int compare = CHINA_COLLATOR.compare(s1, s2);
			if (compare != 0) {
				return compare;
			}
			
			if (!find1 && !find2) {
				return 0;
			}
			if (!find1 || !find2) {
				return find1 ? 1 : 0;
			}
			
			BigInteger i1 = new BigInteger(matcher1.group());
			BigInteger i2 = new BigInteger(matcher2.group());
			compare = i1.compareTo(i2);
			if (compare != 0) {
				return compare;
			}
			
			end1 = matcher1.end();
			end2 = matcher2.end();
		}
	}
	
	public static void main(String[] args) {
		
		System.out.println(new BigInteger("99999999999999999999999999999999999999999999999999999999999999999999999999999999999"));
		
//		String s = "aa12.3asd45.6bb";
//		Matcher matcher1 = PATTERN.matcher(s);
//		while (matcher1.find()) {
//			System.out.println(">>>>" + matcher1.group());
//			System.out.println(matcher1.start());
//			System.out.println(matcher1.end());
//			System.out.println();
//		}
	}
	
}
